'use strict';

var ContextDependencyHelpers = exports;

/**
 * Escapes regular expression metacharacters
 * @param {string} str String to quote
 * @return {string} Escaped string
 */
function quotemeta(str) {
  return str.replace(/[-[\]\\/{}()*+?.^$|]/g, '\\$&');
}

ContextDependencyHelpers.create = function (Dep, range, param, expr, options, chunkName) {
  if (param.isWrapped() && param.prefix && param.prefix.isString()) {
    var prefix = param.prefix.string;
    var prefixRange = param.prefix.range;
    var valueRange = [prefixRange ? prefixRange[1] : param.range[0], param.range[1]];
    var idx = prefix.lastIndexOf('/');
    var context = '.';
    if (idx >= 0) {
      context = prefix.substr(0, idx);
      prefix = `.${prefix.substr(idx)}`;
    }
    var regExp = new RegExp(`^${quotemeta(prefix)}.*$`);
    var dep = new Dep(context, options.wrappedContextRecursive, regExp, range, valueRange, chunkName);
    dep.loc = expr.loc;
    dep.prepend = param.prefix && param.prefix.isString() ? prefix : null;
    dep.critical = options.wrappedContextCritical && 'a part of the request of a dependency is an expression';
    return dep;
  }
  return null;
};