'use strict';

var async = require('async');
var Tapable = require('tapable');
var OpenUI5ResourceModule = require('./OpenUI5ResourceModule');

class OpenUI5ResourceModuleFactory extends Tapable {
  constructor(resolvers) {
    super();
    this.resolvers = resolvers;
  }

  create(data, callback) {
    var _this = this;

    var dependencies = data.dependencies;
    var dependency = dependencies[0];

    this.applyPluginsAsyncWaterfall('before-resolve', {
      context: dependency.context,
      modulePath: dependency.modulePath,
      extensions: dependency.extensions,
      libraries: dependency.libraries,
      translations: dependency.translations,
      locales: dependency.locales ? dependency.locales : dependency.translations,
      failOnError: dependency.failOnError
    }, function (err, result) {
      var context = result.context;
      var modulePath = result.modulePath;
      var extensions = result.extensions;
      var libraries = result.libraries;
      var translations = result.translations;
      var locales = result.locales;
      var failOnError = result.failOnError;
      var resolvers = _this.resolvers;

      var messagebundles = ['messagebundle.properties'];
      translations.forEach(function (translation) {
        messagebundles.push(`messagebundle_${translation}.properties`);
      });

      var resources = [];
      libraries.forEach(function (library) {
        var libSlash = library.replace(/\./g, '/');
        messagebundles.forEach(function (messagebundle) {
          resources.push(`${libSlash}/${messagebundle}`);
        });
      });
      locales.forEach(function (translation) {
        resources.push(`sap/ui/core/cldr/${translation}.json`);
      });

      async.map(resources, function (resource, callback) {
        resolvers.normal.resolve({}, dependency.context, resource, function (err) {
          if (err) {
            callback(err);
            return;
          }
          callback(null, resource);
        });
      }, function (err, result) {
        if (err) {
          callback(err);
          return;
        }

        _this.applyPluginsAsyncWaterfall('after-resolve', {
          context,
          modulePath,
          extensions,
          resources: result,
          failOnError
        }, function (err, result) {
          if (err) return callback(err);

          // Ignored
          if (!result) return callback();

          return callback(null, new OpenUI5ResourceModule(result.context, result.modulePath, result.extensions, result.resources, result.failOnError));
        });
      });
    });
  }
}

module.exports = OpenUI5ResourceModuleFactory;